#include <vector>
#include <algorithm>
#include <set>

#include<vcg/simplex/vertexplus/base.h>
#include<vcg/simplex/faceplus/base.h>
#include<vcg/complex/trimesh/base.h>

#include<vcg/complex/trimesh/update/flag.h>

#include<wrap/io_trimesh/import_ply.h>

using namespace vcg;

class MyEdge;    // dummy prototype never used
class MyFace;
class MyVertex;

class MyVertex  : public VertexSimp2< MyVertex, MyEdge, MyFace, vertex::Coord3f, vertex::BitFlags  >{};
class MyFace    : public FaceSimp2  < MyVertex, MyEdge, MyFace, face::VertexRef, face::BitFlags > {};

class MyMesh : public tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace > >{};


int main(int argc,char **argv ){

	MyMesh m;

	if(tri::io::ImporterPLY<MyMesh>::Open(m,argv[1])!=0)
		{
      printf("Error reading file  %s\n",argv[1]);
			exit(0);
		}
		
	printf("vn %i fn %i\n",m.vn, m.fn);
	printf("vn %i fn %i\n",m.vert.size(), m.face.size());

	tri::UpdateFlags<MyMesh>::FaceBorderFromNone(m);
	int borderEdgeCnt=0;
	for(int i=0;i<m.face.size();++i)
	{
		if(!m.face[i].IsD())
		{
			for(int j=0;j<3;++j)
			{
				if(m.face[i].IsB(j)) 
					borderEdgeCnt++;
			}
		}
	}
	
	printf("Counted Border edges %i\n",borderEdgeCnt);
  
	printf("edge number %i\n", ( m.fn *3 + borderEdgeCnt)/2);
	
	std::set<std::pair<MyVertex *,MyVertex *> > edgeSet;
	
	for(int i=0;i<m.face.size();++i)
	{
		if(!m.face[i].IsD())
		{
			for(int j=0;j<3;++j)
			{
				if( m.face[i].V0(j) < m.face[i].V1(j))
							edgeSet.insert(std::make_pair( m.face[i].V0(j) , m.face[i].V1(j)  ));
						else
							edgeSet.insert(std::make_pair( m.face[i].V1(j) , m.face[i].V0(j)  ));

			}
		}
	}

	printf("Number of edges %i\n",edgeSet.size());
	
	printf(" Vn - en + fn = %i\n", m.vn - edgeSet.size() + m.fn);
	
  return 0;
}

